#ifndef __FBVIDEO_CLASS__
#define __FBVIDEO_CLASS__

#include once "windows.bi"
#include once "win\commdlg.bi"
#include once "win\vfw.bi"

#include "fbgfx.bi"

#ifndef __FB_WIN32__
#error 666: Target must be Windows 32/64-bit !
#endif

enum SUPORTEDCOMPRESSION
  RGB24 = 0
  YV12  = &H32315659 ' 12 bit 8Y plane followed by 8bit 2x2 subsampled V and U planes.
  I420  = &H30323449 ' 12 bit same as YV12 but U V in reverse order.
  IYUV  = &H56555949 ' same as i420
  YUV2  = &H32565559 ' YUV 4:2:2 (UYVY but with different ordering)
  YUY2  = &H32595559
End enum

Type ImageProc_t   As Sub (Byval lpImage As Any Ptr)

Type ErrorProc_t   As Function ( _
  Byval hWin       As HWND, _
  Byval nID        As long, _
  Byval lpszText   As Zstring Ptr) As LRESULT

Type StatusProc_t As Function ( _
  Byval hWin      As HWND, _
  Byval nID       As long, _
  Byval lpszText  As Zstring Ptr) As LRESULT

Type ControlProc_t As Function ( _
  Byval hWin   As HWND, _
  Byval nState As long) As LRESULT

Type YieldProc_t As Function( _
  Byval hWin As HWND) As LRESULT

Type VideoFrameProc_t As Function ( _
  Byval hWin    As HWND, _
  Byval pHeader As PVIDEOHDR) As LRESULT

Type AudioStreamProc_t As Function ( _
  Byval hWin    As HWND, _
  Byval pHeader As LPWAVEHDR) As LRESULT

Type VideoStreamProc_t As Function ( _
  Byval hWin    As HWND, _
  Byval pHeader As PVIDEOHDR) As LRESULT


Declare Function ConvertRGB (Byval pHeader As PVIDEOHDR) As Any Ptr

Declare Function ConvertI420(Byval pHeader As PVIDEOHDR) As Any Ptr

Declare Function ConvertYUV2(Byval pHeader As PVIDEOHDR) As Any Ptr

Declare Function ConvertYUY2(Byval pHeader As PVIDEOHDR) As Any Ptr

Declare Function ConvertNone(Byval pHeader As PVIDEOHDR) As Any Ptr

Dim Shared As Integer _nConnectedDevices

' deligate in user space
Dim Shared As ErrorProc_t       _UserErrorProc
Dim Shared As StatusProc_t      _UserStatusProc
Dim Shared As ControlProc_t     _UserControlProc
Dim Shared As YieldProc_t       _UserYieldProc
Dim Shared As VideoFrameProc_t  _UserVideoFrameProc
Dim Shared As AudioStreamProc_t _UserAudioStreamProc
Dim Shared As VideoStreamProc_t _UserVideoStreamProc
Dim Shared As ImageProc_t       _UserImageProc

Dim Shared As BITMAPINFO _Videoformat
Dim Shared As FB.PUT_HEADER Ptr _pImage

Function _ErrorProc ( _
  Byval hWin       As HWND, _
  Byval nID        As long, _
  Byval lpszText   As Zstring Ptr) As LRESULT
  If _UserErrorProc<>NULL Then
    Return _UserErrorProc(hWin,nID,lpszText)
  End If
  Return True
End Function

Function _StatusProc ( _
  Byval hWin      As HWND, _
  Byval nID       As long, _
  Byval lpszText  As Zstring Ptr) As LRESULT
  If _UserStatusProc<>NULL Then
    Return _UserStatusProc(hWin,nID,lpszText)
  End If
  Return True
End Function

Function _ControlProc ( _
  Byval hWin   As HWND, _
  Byval nState As long) As LRESULT
  If _UserControlProc<>NULL Then
    Return _UserControlProc(hWin,nState)
  End If
  Return True
End Function

Function _YieldProc ( _
  Byval hWin As HWND) As LRESULT
  If _UserYieldProc<>NULL Then
    Return _UserYieldProc(hWin)
  End If
  Return True
End Function

Function _VideoStreamProc ( _
  Byval hWin    As HWND, _
  Byval pHeader As PVIDEOHDR) As LRESULT

  If (_UserImageProc<>NULL) AndAlso (_pImage<>NULL) Then
    Select Case _VideoFormat.bmiHeader.biCompression
    Case RGB24
      _UserImageProc(ConvertRGB(pHeader))
    Case I420,IYUV
      _UserImageProc(ConvertI420(pHeader))
    Case YUV2
      _UserImageProc(ConvertYUV2(pHeader))
    Case YUY2
      _UserImageProc(ConvertYUY2(pHeader))
    Case Else
      _UserImageProc(ConvertNone(pHeader))
    End Select
  End If
  If _UserVideoStreamProc<>NULL Then
    Return _UserVideoStreamProc(hWin,pHeader)
  End If
  Return True
End Function

Function _AudioStreamProc (_
  Byval hWin    As HWND, _
  Byval pHeader As LPWAVEHDR) As LRESULT
  If _UserAudioStreamProc<>NULL Then
    Return _UserAudioStreamProc(hWin,pHeader)
  End If
  Return True
End Function

Type VIDEOIN
  Public:
  Declare Constructor (Byval AutoMode As BOOLEAN=False)
  Declare Destructor
  Declare Property IsCreated          As BOOLEAN
  Declare Property IsConnected        As BOOLEAN
  Declare Property IsCapturing        As BOOLEAN
  Declare Property CaptureInitialized As BOOLEAN 
   
  Declare Property Width              As Integer
  Declare Property Height             As Integer
  Declare Property BPP                As Integer
  Declare Property Compression        As Integer

  Declare Property Rate               As Integer

  Declare Sub ShowDlgVideoSource
  Declare Sub ShowDlgVideoFormat
  Declare Sub ShowDlgVideoCompression
  Declare Sub ShowDlgVideoDisplay

  Declare Sub Create
  Declare Sub Connect
  Declare Sub Disconnect
  Declare Sub Destroy
  Declare Sub Start
  Declare Sub Stop
  Declare Sub SaveImage         (Byval File As String)

  Declare Sub GetStatus         (Byref s As CAPSTATUS)
  Declare Sub GetParams         (Byref p As CAPTUREPARMS)
  Declare Sub SetParams         (Byval p As CAPTUREPARMS)
  Declare Sub GetVideoFormat    (Byref v As BITMAPINFO)
  Declare Sub SetVideoFormat    (Byval v As BITMAPINFO)

  Declare Sub SetErrorProc      (Byval cb As ErrorProc_t)
  Declare Sub SetStatusProc     (Byval cb As StatusProc_t)
  Declare Sub SetControlProc    (Byval cb As ControlProc_t)
  Declare Sub SetYieldProc      (Byval cb As YieldProc_t)
  Declare Sub SetImageProc      (Byval cb As ImageProc_t)
  Declare Sub SetAudioStreamProc(Byval cb As AudioStreamProc_t)
  Declare Sub SetVideoStreamProc(Byval cb As VideoStreamProc_t)

  Private:
  As HWND          _hDev
  As HWND          _hWin
  As CAPDRIVERCAPS _Caps
  As CAPTUREPARMS  _Params
  As CAPSTATUS     _Status

' As CAP_AUDIO_FORMAT_t  _Audioformat
  As String        _DIBFile
  as Integer       _Rate
  As BOOLEAN       _IsConnected,_IsCapturing,_IsCreated,_IsPreview

  'Declare Function _InitDll As BOOLEAN
  Declare Sub _Create
  Declare Sub _Destroy
  Declare Sub _Connect
  Declare Sub _Start
  Declare Sub _Stop
  Declare Sub _Disconnect
  Declare Sub _EnableCallbacks
  Declare Sub _DisableCallbacks

  Declare Sub _GetCaps
  Declare Sub _GetStatus
  Declare Sub _GetParams
  Declare Sub _SetParams
  Declare Sub _GetVideoFormat
  Declare Sub _SetVideoFormat
  Declare Sub _GetAudioFormat
  Declare Sub _SetAudioFormat
  Declare Sub _SaveDIB
End Type

Private _
Sub VIDEOIN._Create
  dim as integer iWin 
  If _IsCreated=True Then _Destroy
  If ScreenPtr<>NULL Then
    ScreenControl FB.GET_WINDOW_HANDLE,iWin
    _hWin=cast(HWND,iWin)
  Else
  #ifndef __FB_64BIT__
    _hWin = FindWindow(0,0)
  #else
    ScreenRes 640,480,32
    ScreenControl FB.GET_WINDOW_HANDLE,iWin
    _hWin=cast(HWND,iWin)  
  #endif  
  Endif
  ' print "hWin = " &  _hWin
  ' print _VideoFormat.bmiHeader.biWidth & " x " & _VideoFormat.bmiHeader.biHeight
   ' or WS_VISIBLE
  _hDev=capCreateCaptureWindow("VIDEOIN:",WS_CHILD,0,0,_VideoFormat.bmiHeader.biWidth,_VideoFormat.bmiHeader.biHeight,_hWin,1)
  If _hDev=False Then
    _IsCreated=False
     MessageBox 0," can't create capture window !","ERROR:",MB_ICONERROR
  Else
    _EnableCallbacks
    _IsCreated=True
  End If
End Sub


Sub VIDEOIN._Start
  If _IsConnected=False Then Return
  If _IsCapturing=True  Then _stop
  If (_VideoFormat.bmiHeader.biWidth<>0) Andalso (_VideoFormat.bmiHeader.biHeight<>0) Andalso (ScreenPtr<>NULL) Then
    If (_pImage<>NULL) Then ImageDestroy(_pImage)
    _pImage=ImageCreate(_VideoFormat.bmiHeader.biWidth,Abs(_VideoFormat.bmiHeader.biHeight))
  End If
  _IsCapturing=SendMessage(_hDev,WM_CAP_SEQUENCE_NOFILE,0,0)
End Sub

Sub VIDEOIN._Destroy
  If _IsCreated=False  Then Return
  If _IsCapturing=True Then _Stop
  If _IsConnected=True Then _Disconnect
  DestroyWindow(_hDev):_hDev=0
  _IsCreated=False
End Sub
sub VIDEOIN._stop
  If _IsCapturing=False Then Return
  SendMessage _hDev,WM_CAP_STOP,0,0
  _IsCapturing=False
End Sub
sub VIDEOIN._disconnect
  If _IsConnected=False Then Return
  If _IsCapturing=True  Then _stop
  SendMessage _hDev,WM_CAP_DRIVER_DISCONNECT,0,0
  _nConnectedDevices-=1
  _IsConnected=False
End Sub
sub VIDEOIN._connect
  Dim As BOOLEAN ret
  If _IsCreated  =False Then Return
  If _IsConnected=True  Then Return
  ret=SendMessage(_hDev,WM_CAP_DRIVER_CONNECT,0,0)
  If ret<>True Then ret=False
  If ret=True  Then _nConnectedDevices+=1
  _IsConnected=ret
End Sub

Sub VIDEOIN._EnableCallbacks
  Dim As Integer ret
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_CAPCONTROL, 0,cast(LPARAM,@_ControlProc))
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_STATUS,     0,cast(LPARAM,@_StatusProc))
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_ERROR,      0,cast(LPARAM,@_ErrorProc))
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_YIELD,      0,cast(LPARAM,@_YieldProc))
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_VIDEOSTREAM,0,cast(LPARAM,@_VideoStreamProc))
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_WAVESTREAM ,0,cast(LPARAM,@_AudioStreamProc))
End Sub
sub VIDEOIN._DisableCallbacks
  Dim As Integer ret
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_STATUS      ,0,NULL)
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_ERROR       ,0,NULL)
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_YIELD       ,0,NULL)
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_VIDEOSTREAM ,0,NULL)
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_WAVESTREAM  ,0,NULL)
  ret=SendMessage(_hDev,WM_CAP_SET_CALLBACK_CAPCONTROL  ,0,NULL)
End Sub


Sub VIDEOIN._GetCaps
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_DRIVER_GET_CAPS,sizeof(CAPSTATUS),cast(LPARAM,@_Status))
End Sub
sub VIDEOIN._GetStatus
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_GET_STATUS,sizeof(CAPDRIVERCAPS),cast(LPARAM,@_Caps))
End Sub
sub VIDEOIN._GetParams
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_GET_SEQUENCE_SETUP,sizeof(CAPTUREPARMS),cast(LPARAM,@_Params))
End Sub
sub VIDEOIN._SetParams
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_SET_SEQUENCE_SETUP,sizeof(CAPTUREPARMS),cast(LPARAM,@_Params))
End Sub
sub VIDEOIN._GetVideoFormat
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_GET_VIDEOFORMAT,sizeof(BITMAPINFO),cast(LPARAM,@_VideoFormat))
End Sub
sub VIDEOIN._SetVideoFormat
  Dim As Integer ret
  If _IsCreated=False Then Return
  ret=SendMessage(_hDev,WM_CAP_SET_VIDEOFORMAT,sizeof(BITMAPINFO),cast(LPARAM,@_VideoFormat))
  _GetVideoFormat
End Sub
sub VIDEOIN._GetAudioFormat
  If _IsConnected=False Then Return
End Sub
sub VIDEOIN._SetAudioFormat
  If _IsConnected=False Then Return
End Sub

Sub VIDEOIN._SaveDIB
  Dim As Integer ret
  If _IsCreated=False Then Return
  If Len(_DIBFile)=0 Then _DIBFile="tmp.bmp"
  ret=SendMessage(_hDev,WM_CAP_FILE_SAVEDIB, 0,cast(LPARAM,strptr(_DibFile)))
End Sub


Constructor VIDEOIN (Byval AutoMode As BOOLEAN)
  If AutoMode=True Then
    _VideoFormat.bmiHeader.biWidth=320:_VideoFormat.bmiHeader.biHeight=240
    ' no preview
    _Create
    If _IsCreated=True Then
      _Connect
      While _IsConnected=False
        var ret = MessageBox(_hWin,"Can't connect the video device [" & Str(_nConnectedDevices+1) & "] !" & Chr(13,10) & _
                                   "The equipment should be attached and switched on."   & Chr(13,10,13,10) & _ 
                                   "Connect the device and try it again.", "Info:",MB_ICONINFORMATION Or MB_RETRYCANCEL)
        If ret=2 Then Return
        _Connect
      Wend
      _GetCaps
      _GetVideoFormat
      If _VideoFormat.bmiHeader.biCompression<>0 Then 
        ' first try RGB24
        _VideoFormat.bmiHeader.biCompression=0
        _VideoFormat.bmiHeader.biBitcount   =24
        _VideoFormat.bmiHeader.biPlanes     =1
        _VideoFormat.bmiHeader.biSizeImage  =_VideoFormat.bmiHeader.biWidth*_VideoFormat.bmiHeader.biWidth
        _VideoFormat.bmiHeader.biSizeImage* =3
        _SetVideoFormat
        If _VideoFormat.bmiHeader.biCompression<>0 Then 
          While (_VideoFormat.bmiHeader.biCompression<>RGB24) AndAlso _
                (_VideoFormat.bmiHeader.biCompression<>i420 ) AndAlso _
                (_VideoFormat.bmiHeader.biCompression<>IYUV ) AndAlso _
                (_VideoFormat.bmiHeader.biCompression<>YUV2 ) AndAlso _
                (_VideoFormat.bmiHeader.biCompression<>YV12 ) AndAlso _
                (_VideoFormat.bmiHeader.biCompression<>YUY2 )
                var ret = MessageBox(_hWin, _
                "Video format raw RGB24 not supported." & Chr(13,10) & _
                "Please select in next dialog one of the following formats."   & Chr(13,10,13,10) & _ 
                "Any RGB format or compressed I420, IYUV, YV12, YUV2, YUY2", "Info:",MB_ICONINFORMATION Or MB_RETRYCANCEL)
                If ret=2 Then Exit While
             SendMessage(_hDev, WM_CAP_DLG_VIDEOFORMAT, 0, 0)
             _GetVideoFormat
          Wend
        End If
      End If
      _GetParams
      _Params.dwRequestMicroSecPerFrame=1000000\20
      _Params.fMakeUserHitOKToCapture=False
      _Params.fYield                 =True
      _Params.fCaptureAudio          =False
      _Params.fAbortLeftMouse        =False
      _Params.fAbortRightMouse       =False
      _SetParams
    End If
  End If
End Constructor

' stop any captureing
' disconnect the device
' free the library
Destructor VIDEOIN
  Dim As Integer ret
  If _pImage    <>NULL Then ImageDestroy(_pImage):_pImage=0
  If _IsCapturing=True Then _stop
  If _IsConnected=True Then _disconnect
  If _IsCreated  =True Then _destroy
End Destructor

' user properties
Property  VIDEOIN.IsCreated As BOOLEAN
  Return _IsCreated
End Property
property  VIDEOIN.IsConnected As BOOLEAN
  Return _IsConnected
End Property
property  VIDEOIN.IsCapturing As BOOLEAN
  Return _IsCapturing
End Property
Property VIDEOIN.CaptureInitialized As BOOLEAN
  Return _Caps.fCaptureInitialized
End Property

Sub VIDEOIN.ShowDlgVideoSource
  If _IsCreated=False Then Return
  SendMessage(_hDev, WM_CAP_DLG_VIDEOSOURCE, 0, 0)
End Sub
sub VIDEOIN.ShowDlgVideoFormat
  If _IsCreated=False Then Return
  SendMessage(_hDev, WM_CAP_DLG_VIDEOFORMAT, 0, 0)
  _getVideoFormat

  if _VideoFormat.bmiHeader.biWidth=0 or _VideoFormat.bmiHeader.biHeight=0 then
    _VideoFormat.bmiHeader.biWidth=320:_VideoFormat.bmiHeader.biHeight=240 ' !!!
    _VideoFormat.bmiHeader.biCompression=-1
  end if
End Sub
sub VIDEOIN.ShowDlgVideoDisplay
  If _IsCreated=False Then Return
  SendMessage(_hDev, WM_CAP_DLG_VIDEODISPLAY, 0, 0)
End Sub
sub VIDEOIN.ShowDlgVideoCompression
   SendMessage(_hDev, WM_CAP_DLG_VIDEOCOMPRESSION, 0, 0)
End Sub 
Property VIDEOIN.Width As Integer
  Return _VideoFormat.bmiHeader.biWidth
End Property
Property VIDEOIN.Height As Integer
  Return _VideoFormat.bmiHeader.biHeight
End Property
Property VIDEOIN.BPP As Integer
  Return _VideoFormat.bmiHeader.biBitCount
End Property
Property VIDEOIN.Compression As Integer
  Return _VideoFormat.bmiHeader.biCompression
End Property

Property VIDEOIN.Rate As Integer
  Return _Rate
End Property

' user methods
Sub VIDEOIN.SaveImage(Byval File As String)
  _DIBFile=File
  _SaveDIB
End Sub

Sub VIDEOIN.Create
  _Create
End Sub
Sub VIDEOIN.Start
  _Start
End Sub
Sub VIDEOIN.Stop
  _Stop
End Sub
Sub VIDEOIN.Connect
  _Connect
End Sub
Sub VIDEOIN.Disconnect
  _Disconnect
End Sub
Sub VIDEOIN.Destroy
  _Destroy
End Sub

Sub VIDEOIN.GetParams(Byref p As CAPTUREPARMS)
  _getParams
  p=_Params
End Sub
sub VIDEOIN.SetParams(Byval p As CAPTUREPARMS)
  _Params=p
  _setParams
End Sub
sub VIDEOIN.GetVideoFormat(Byref v As BITMAPINFO)
  _GetVideoFormat
  v=_VideoFormat
End Sub
sub VIDEOIN.SetVideoFormat(Byval v As BITMAPINFO)
  _VideoFormat=v
  _setVideoFormat
End Sub
sub VIDEOIN.GetStatus(Byref s As CAPSTATUS)
  _GetStatus
  s=_Status
End Sub

Sub VIDEOIN.SetErrorProc      (Byval cb As ErrorProc_t)
  _UserErrorProc=cb
End Sub
sub VIDEOIN.SetStatusProc     (Byval cb As StatusProc_t)
  _UserStatusProc=cb
End Sub
sub VIDEOIN.SetControlProc    (Byval cb As ControlProc_t)
  _UserControlProc=cb
End Sub
sub VIDEOIN.SetYieldProc      (Byval cb As YieldProc_t)
  _UserYieldProc=cb
End Sub
sub VIDEOIN.SetImageProc      (Byval cb As ImageProc_t)
  _UserImageProc=cb
End Sub
sub VIDEOIN.SetAudioStreamProc(Byval cb As AudioStreamProc_t)
  _UserAudioStreamProc=cb
End Sub
sub VIDEOIN.SetVideoStreamProc(Byval cb As VideoStreamProc_t)
  _UserVideoStreamProc=cb
End Sub

' converter
Function ConvertRGB (Byval pHeader As PVIDEOHDR) As Any Ptr
  
  If pHeader->dwBytesUsed = _VideoFormat.bmiHeader.biSizeImage Then
    Dim As integer w,h,spitch,dpitch
    Dim As Any Ptr src
    Dim As Any Ptr des
    w=    _VideoFormat.bmiHeader.biWidth
    h=Abs(_VideoFormat.bmiHeader.biHeight)
    spitch=w*3
    dpitch=(w*4+15):dpitch and=&HFFF0
    src =pHeader->lpData
    src+=spitch*(h-1)
    des =_pImage
    des+=32
    for y as integer = 0 to h-1
      dim as ubyte ptr s = src
      dim as ubyte ptr d = des 
      for x as integer = 0 to w-1
        d[0] = s[0]
        d[1] = s[1]
        d[2] = s[2]
        d[3] = &HFF
        d+=4
        s+=3
      next
      src-=spitch
      des+=dpitch 
    next
  End If
  Return _pImage
End Function

Function ConvertI420(Byval pHeader As PVIDEOHDR) As Any Ptr
  Dim As Integer   i,i1,i2,i3,iU=1,iV=1
  Dim As Integer   xres =_VideoFormat.bmiHeader.biWidth
  Dim As Integer   xres2=xres Shr 1
  Dim As Integer   yres =_VideoFormat.bmiHeader.biHeight
  Dim As Integer   nPixels=_VideoFormat.bmiHeader.biWidth*_VideoFormat.bmiHeader.biHeight
  Dim As Integer   offset1=nPixels
  Dim As Integer   offset2=nPixels+(nPixels Shr 2)
  Dim As Ulong     col
  Dim As Single    y,u,v
  Dim As Single    rr,gg,bb
  Dim As Ubyte     r,g,b
  Dim As Ubyte Ptr src = pHeader->lpData
  Dim As ulong Ptr des = cptr(ulong Ptr,_pImage)
  '? "i420"
  des+=8
  For i=0 To nPixels-1
    y=Csng(src[i])
    u=Csng(src[offset1+i2+i3])-127.5f
    v=Csng(src[offset2+i2+i3])-127.5f
    rr=y+v
    gg=y-(u+v)
    bb=y+u
    If rr<0 Then
      r=0
    Elseif rr>255 Then
      r=255
    Else
      r=cubyte(rr)
    End If
    If gg<0 Then
      g=0
    Elseif gg>255 Then
      g=255
    Else
      g=cubyte(gg)
    End If
    If bb<0 Then
      b=0
    Elseif bb>255 Then
      b=255
    Else
      b=cubyte(bb)
    End If
    des[i]=RGB(r,g,b)
    iU Xor=1:If (iU) Then i2+=1
    i1+=1
    If (i1=xres) Then
      i1=0:i2=0
      iV Xor=1
      If(iV) Then i3+=xres2
    End If
  Next
  Return _pImage
End Function

Function ConvertYUV2(Byval pHeader As PVIDEOHDR) As Any Ptr
  Dim As Integer   i,n,t
  Dim As Integer   nPixels=_VideoFormat.bmiHeader.biWidth*_VideoFormat.bmiHeader.biHeight
  Dim As ulong     col
  Dim As Single    y,y2,u,v
  Dim As Single    rr,gg,bb
  Dim As Ubyte     r,g,b
  Dim As Ubyte Ptr src = pHeader->lpData
  Dim As ulong Ptr des = cptr(ulong Ptr,_pImage)
  '? "YUV2"
  des+=8
  nPixels shr=1
  For n=0 To nPixels-1
    y =Csng(src[i+0])
    u =Csng(src[i+1])-127.5f
    y2=Csng(src[i+3])
    v =Csng(src[i+2])-127.5f

    rr=y+v:gg=y-(u+v):bb=y+u
    If rr<0 Then
      r=0
    Elseif rr>255 Then
      r=255
    Else
      r=cubyte(rr)
    End If
    If gg<0 Then
      g=0
    Elseif gg>255 Then
      g=255
    Else
      g=cubyte(gg)
    End If
    If bb<0 Then
      b=0
    Elseif bb>255 Then
      b=255
    Else
      b=cubyte(bb)
    End If
    des[t]=RGB(r,g,b):t+=1
    ' second pixel
    rr=y2+v:gg=y2-(u+v):bb=y2+u
    If rr<0 Then
      r=0
    Elseif rr>255 Then
      r=255
    Else
      r=cubyte(rr)
    End If
    If gg<0 Then
      g=0
    Elseif gg>255 Then
      g=255
    Else
      g=cubyte(gg)
    End If
    If bb<0 Then
      b=0
    Elseif bb>255 Then
      b=255
    Else
      b=cubyte(bb)
    End If
    des[t]=RGB(r,g,b)
    t+=1
    i+=4
  Next
  Return _pImage
End Function

Function ConvertYUY2(Byval pHeader As PVIDEOHDR) As Any Ptr
  Dim As Integer   i,n,t
  Dim As Integer   nPixels=_VideoFormat.bmiHeader.biWidth*_VideoFormat.bmiHeader.biHeight
  Dim As ulong     col
  Dim As Single    y,y2,u,v
  Dim As Single    rr,gg,bb,c,d,e
  Dim As Ubyte     r,g,b
  Dim As Ubyte Ptr src = pHeader->lpData
  Dim As ulong Ptr des = cptr(ulong Ptr,_pImage)
  '? "YUY2"
  des+=8
  nPixels shr=1
   For n=0 To nPixels-1
    #if 1 
    y =Csng(src[i+0])-16
    u =Csng(src[i+1])-128
    y2=Csng(src[i+2])-16
    v =Csng(src[i+3])-128
    C *= 1.164383
    rr = C                + 1.596027 * V
    gg = C - 0.391762 * U - 0.812968 * V
    bb = C + 2.017232 * U
    If rr<0 Then
      r=0
    Elseif rr>255 Then
      r=255
    Else
      r=cubyte(rr)
    End If
    If gg<0 Then
      g=0
    Elseif gg>255 Then
      g=255
    Else
      g=cubyte(gg)
    End If
    If bb<0 Then
      b=0
    Elseif bb>255 Then
      b=255
    Else
      b=cubyte(bb)
    End If
    des[t]=RGB(r,g,b)
    #else
    r=src[i]
    des[t]=RGB(r,r,r)
    #endif
    t+=1

    #if 1
    ' second pixel
    'rr=y2+v:gg=y2-(u+v):bb=y2+u
    C = Y2
    C *= 1.164383
    rr = C                + 1.596027 * V
    gg = C - 0.391762 * U - 0.812968 * V
    bb = C + 2.017232 * U
    If rr<0 Then
      r=0
    Elseif rr>255 Then
      r=255
    Else
      r=cubyte(rr)
    End If
    If gg<0 Then
      g=0
    Elseif gg>255 Then
      g=255
    Else
      g=cubyte(gg)
    End If
    If bb<0 Then
      b=0
    Elseif bb>255 Then
      b=255
    Else
      b=cubyte(bb)
    End If
    des[t]=RGB(r,g,b)
    #else
    r=src[i+2]
    des[t]=RGB(r,r,r)
    #endif
    t+=1
    i+=4
  Next
  Return _pImage
End Function

function ConvertNone(Byval pHeader As PVIDEOHDR) As Any Ptr
  static as integer frames,y=-8
  If (_pImage<>NULL) andalso (pHeader<>NULL) Then
    line _pImage,(0,0)-(_pImage->width-1,_pImage->height-1),RGB(255,0,255),BF
    Draw String _pImage,(0,y), "frame [" & str(frames) & "] no converter!",rgb(255,255,255)
    y+=1:frames+=1
    if y>=_pImage->Height then y=-8
  end if
  Return _pImage
End Function

#endif ' __FBVIDEO_CLASS__ 
 
